--[[

--]]
--
-- An FRSKY S.Port <passthrough protocol> based Telemetry script for the Horus X10 and X12 radios
--
-- Copyright (C) 2018-2019. Alessandro Apostoli
-- https://github.com/yaapu
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY, without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, see <http://www.gnu.org/licenses>.
--

---------------------
-- MAIN CONFIG
-- 480x272 LCD_W x LCD_H
---------------------

---------------------
-- VERSION
---------------------
-- load and compile of lua files
--#define LOADSCRIPT
-- uncomment to force compile of all chunks, comment for release
--#define 
-- fix for issue OpenTX 2.2.1 on X10/X10S - https://github.com/opentx/opentx/issues/5764

---------------------
-- MAVLITE CONFIG
---------------------

---------------------
-- DEV FEATURE CONFIG
---------------------
-- cache tuning pages
-- cache params pages
--#define CACHE_PARAMS
-- enable full telemetry debug
--#define TELEMETRY_DEBUG
-- enable full telemetry decoding
--#define FULL_TELEMETRY
-- enable memory debuging 
-- enable dev code
--#define DEV
-- use radio channels imputs to generate fake telemetry data
--#define TESTMODE

---------------------
-- DEBUG REFRESH RATES
---------------------
-- calc and show hud refresh rate
-- calc and show telemetry process rate



--------------------------------------------------------------------------------
-- MENU VALUE,COMBO
--------------------------------------------------------------------------------

-----------------------
-- LIBRARY LOADING
-----------------------

--[[
  status of pending mavlite messages
]]






local frameTypes = {}
local frameType = nil

--[[
	MAV_TYPE_GENERIC=0,               /* Generic micro air vehicle. | */
	MAV_TYPE_FIXED_WING=1,            /* Fixed wing aircraft. | */
	MAV_TYPE_QUADROTOR=2,             /* Quadrotor | */
	MAV_TYPE_COAXIAL=3,               /* Coaxial helicopter | */
	MAV_TYPE_HELICOPTER=4,            /* Normal helicopter with tail rotor. | */
	MAV_TYPE_ANTENNA_TRACKER=5,       /* Ground installation | */
	MAV_TYPE_GCS=6,                   /* Operator control unit / ground control station | */
	MAV_TYPE_AIRSHIP=7,               /* Airship, controlled | */
	MAV_TYPE_FREE_BALLOON=8,          /* Free balloon, uncontrolled | */
	MAV_TYPE_ROCKET=9,                /* Rocket | */
	MAV_TYPE_GROUND_ROVER=10,         /* Ground rover | */
	MAV_TYPE_SURFACE_BOAT=11,         /* Surface vessel, boat, ship | */
	MAV_TYPE_SUBMARINE=12,            /* Submarine | */
  MAV_TYPE_HEXAROTOR=13,            /* Hexarotor | */
	MAV_TYPE_OCTOROTOR=14,            /* Octorotor | */
	MAV_TYPE_TRICOPTER=15,            /* Tricopter | */
	MAV_TYPE_FLAPPING_WING=16,        /* Flapping wing | */
	MAV_TYPE_KITE=17,                 /* Kite | */
	MAV_TYPE_ONBOARD_CONTROLLER=18,   /* Onboard companion controller | */
	MAV_TYPE_VTOL_DUOROTOR=19,        /* Two-rotor VTOL using control surfaces in vertical operation in addition. Tailsitter. | */
	MAV_TYPE_VTOL_QUADROTOR=20,       /* Quad-rotor VTOL using a V-shaped quad config in vertical operation. Tailsitter. | */
	MAV_TYPE_VTOL_TILTROTOR=21,       /* Tiltrotor VTOL | */
	MAV_TYPE_VTOL_RESERVED2=22,       /* VTOL reserved 2 | */
	MAV_TYPE_VTOL_RESERVED3=23,       /* VTOL reserved 3 | */
	MAV_TYPE_VTOL_RESERVED4=24,       /* VTOL reserved 4 | */
	MAV_TYPE_VTOL_RESERVED5=25,       /* VTOL reserved 5 | */
	MAV_TYPE_GIMBAL=26,               /* Onboard gimbal | */
	MAV_TYPE_ADSB=27,                 /* Onboard ADSB peripheral | */
	MAV_TYPE_PARAFOIL=28,             /* Steerable, nonrigid airfoil | */
	MAV_TYPE_DODECAROTOR=29,          /* Dodecarotor | */
]]
-- copter
frameTypes[0]   = "c"
frameTypes[2]   = "c"
frameTypes[3]   = "c"
frameTypes[4]   = "h"
frameTypes[13]  = "c"
frameTypes[14]  = "c"
frameTypes[15]  = "c"
frameTypes[29]  = "c"
-- plane
frameTypes[1]   = "p"
frameTypes[16]  = "p"
frameTypes[19]  = "q"
frameTypes[20]  = "q"
frameTypes[21]  = "q"
frameTypes[22]  = "q"
frameTypes[23]  = "q"
frameTypes[24]  = "q"
frameTypes[25]  = "q"
frameTypes[28]  = "p"
-- rover
frameTypes[10]  = "r"
-- boat
frameTypes[11]  = "b"

------------------------------
-- TELEMETRY DATA
------------------------------
local telemetry = {}
-- PARAMS
telemetry.frameType = -1

--[[
0	MAV_SEVERITY_EMERGENCY	System is unusable. This is a "panic" condition.
1	MAV_SEVERITY_ALERT	Action should be taken immediately. Indicates error in non-critical systems.
2	MAV_SEVERITY_CRITICAL	Action must be taken immediately. Indicates failure in a primary system.
3	MAV_SEVERITY_ERROR	Indicates an error in secondary/redundant systems.
4	MAV_SEVERITY_WARNING	Indicates about a possible future error if this is not resolved within a given timeframe. Example would be a low battery warning.
5	MAV_SEVERITY_NOTICE	An unusual event has occured, though not an error condition. This should be investigated for the root cause.
6	MAV_SEVERITY_INFO	Normal operational messages. Useful for logging. No action is required for these messages.
7	MAV_SEVERITY_DEBUG	Useful non-operational messages that can assist in debugging. These should not occur during normal operation.
--]]
local mavSeverity = {}

mavSeverity[0]="EMR"
mavSeverity[1]="ALR"
mavSeverity[2]="CRT"
mavSeverity[3]="ERR"
mavSeverity[4]="WRN"
mavSeverity[5]="NOT"
mavSeverity[6]="INF"
mavSeverity[7]="DBG"

--------------------------------
-- STATUS DATA
--------------------------------
local status = {}
-- MESSAGES
status.messages = {}
status.msgBuffer = ""
status.lastMsgValue = 0
status.lastMsgTime = 0
status.lastMessage = nil
status.lastMessageSeverity = 0
status.lastMessageCount = 1
status.messageCount = 0
-- LINK STATUS
status.noTelemetryData = 1
status.hideNoTelemetry = false


local soundFileBasePath = "/SOUNDS/yaapu0"

----------------------
--- COLORS
----------------------

--#define COLOR_LABEL 0x7BCF
--#define COLOR_BG 0x0169




local hudcounter = 0
local hudrate = 0
local hudstart = 0

local maxmem = 0

--------------------------------------------------------------------------------
-- CONFIGURATION MENU
--------------------------------------------------------------------------------
local conf = {
  language = "en"
}

local msgRequestStatus = {
  [0] = "IDLE",
  [1] = "READ_REQ",
  [2] = "SET_REQ",
  [3] = "WAIT",
  [4] = "ERR_EXP",
  [5] = "OK",
  [6] = "ERR_IDX",
  [7] = "ERR_RNG",
}

local msgShortRequestStatus = {
  [0] = "na", -- "na", -- NA
  [1] = "gv", -- "get",-- GV
  [2] = "sv", -- "set",-- SV
  [3] = "wt", -- "wait",-- WT
  [4] = "ex",-- EX
  [5] = "ok", -- OK
  [6] = "va",-- VA
  [7] = "rn",-- RN
}

local utils = {}
local mavLib = {}

--------------------
-- params pages
--------------------
local params = {}
local paramsPages = {}
local modelParams = {}

-- parameters used to check specific vehicle params at boot
local globalParams = {}

--------------------
-- tuning page
--------------------
local tuning = {}
local tuningPages = {}

local basePath = "/SCRIPTS/YAAPU/"
local libBasePath = basePath.."LIB/"

local bitmaps = {}
local blinktime = getTime()
local blinkon = false

local menu  = {
  selectedItem = 1,
  editSelected = false,
  offset = 0,
  wrapOffset = 0, -- changes according to enabled/disabled features and panels
  page = 0,
}

local sportPacket = {
  sensor_id = nil,
  frame_id = nil,
  data_id = nil,
  value = nil
}

local mavlite_message= {
  msgid = -1,
  len = 0,
  payload = {},
  checksum = 0
}

local mavlite_status = {
  parse_state = 0,
  current_rx_seq = 0,
  payload_next_byte = 0
}

local idx = 1 -- index of the current item being processed by processItems()
local page = 1
local subPage = 0

utils.doLibrary = function(filename)
  local f = assert(loadScript(libBasePath..filename..".lua"))
  collectgarbage()
  collectgarbage()
  return f()
end

-----------------------------
-- clears the loaded table 
-- and recovers memory
-----------------------------
utils.clearTable = function(t)
  if type(t)=="table" then
    for i,v in pairs(t) do
      if type(v) == "table" then
        utils.clearTable(v)
      end
      t[i] = nil
    end
  end
  t = nil
  collectgarbage()
  collectgarbage()
  maxmem = 0
end  

local function getItemByName(items,name)
  for idx=1,#items
  do
    if items[idx][1] == name then
      return items[idx]
    end
  end
  return nil
end

utils.drawBottomBar = function()
  lcd.setColor(CUSTOM_COLOR,0x0000)
  -- black bar
  lcd.drawFilledRectangle(0,LCD_H-20+3, LCD_W, 17, CUSTOM_COLOR)
  -- message text
  local msg = status.messages[(status.messageCount + #status.messages) % (#status.messages+1)][1]
  
  if status.messages[(status.messageCount + #status.messages) % (#status.messages+1)][2] < 4 then
    lcd.setColor(CUSTOM_COLOR,lcd.RGB(255,70,0))  
  elseif status.messages[(status.messageCount + #status.messages) % (#status.messages+1)][2] == 4 then
      lcd.setColor(CUSTOM_COLOR,lcd.RGB(255,255,0))  
  else
    lcd.setColor(CUSTOM_COLOR,0xFFFF)
  end
  lcd.drawText(1, LCD_H - 20, msg, CUSTOM_COLOR)
end

local function telemetryEnabled()
  if getRSSI() == 0 then
    status.noTelemetryData = 1
  end
  return status.noTelemetryData == 0
end

local function drawWarning(text)
  lcd.setColor(CUSTOM_COLOR,0xFFFF)
  lcd.drawFilledRectangle(48,74, 384, 84, CUSTOM_COLOR)
  lcd.setColor(CUSTOM_COLOR,0xF800)
  lcd.drawFilledRectangle(50,76, 380, 80, CUSTOM_COLOR)
  lcd.setColor(CUSTOM_COLOR,0xFFFF)
  lcd.drawText(65, 80, text, DBLSIZE+CUSTOM_COLOR)
  lcd.drawText(130, 130, "Yaapu Ardupilot Lua 0.1-dev", SMLSIZE+CUSTOM_COLOR)
end

local function drawBars(items)
  lcd.setColor(CUSTOM_COLOR,0x0000)
  lcd.drawFilledRectangle(0,0, LCD_W, 20, CUSTOM_COLOR)
  lcd.drawRectangle(0, 0, LCD_W, 20, CUSTOM_COLOR)
  local itemIdx = string.format("%d/%d",menu.selectedItem,#items)
  lcd.setColor(CUSTOM_COLOR,0xFFFF)
  lcd.drawText(LCD_W,LCD_H-20+1,itemIdx,CUSTOM_COLOR+RIGHT)
end

local function drawMessageScreen()
  for i=0,#status.messages do
    if  status.messages[(status.messageCount + i) % (#status.messages+1)][2] == 4 then
      lcd.setColor(CUSTOM_COLOR,lcd.RGB(255,255,0))
    elseif status.messages[(status.messageCount + i) % (#status.messages+1)][2] < 4 then
      lcd.setColor(CUSTOM_COLOR,lcd.RGB(255,70,0))  
    else
      lcd.setColor(CUSTOM_COLOR,0xFFFF)
    end
    lcd.drawText(0,2+13*i, status.messages[(status.messageCount + i) % (#status.messages+1)][1],SMLSIZE+CUSTOM_COLOR)
  end
end

local function incMenuItem(items,idx)
  if type(items[idx][2]) == "table" then
    items[idx].value = items[idx].value + 1
    if items[idx].value > #items[idx][2] then
      items[idx].value = 1
    end
  else
    items[idx].value = items[idx].value + items[idx][4]
    if items[idx].value > items[idx][3] then
      items[idx].value = items[idx][3]
    end
  end
end

local function decMenuItem(items,idx)
  if type(items[idx][2]) == "table" then
    items[idx].value = items[idx].value - 1
    if items[idx].value < 1 then
      items[idx].value = #items[idx][2]
    end
  else
    items[idx].value = items[idx].value - items[idx][4]
    if items[idx].value < items[idx][2] then
      items[idx].value = items[idx][2]
    end
  end
end

local function getDecimalCount(num)
  local strNum = tostring(num)
  local pos = string.find(strNum,"%.")
  collectgarbage()
  collectgarbage()
  return pos == nil and 0 or #strNum - pos
end

local function drawPanelItem(panel,idx)
  lcd.setColor(CUSTOM_COLOR,0xFFFF)    
  
  local items = panel.list
  
  lcd.drawText(items[idx].x,items[idx].y, items[idx].label,CUSTOM_COLOR+SMLSIZE)
  
  local flags = SMLSIZE
  if idx == menu.selectedItem then
    if menu.editSelected then
        flags = flags+INVERS+BLINK
    else
      flags = flags+INVERS
    end
  end
  
  if items[idx].value == nil then
    lcd.drawText(items[idx].x+panel.labelWidth,items[idx].y, "--------",flags+CUSTOM_COLOR)
  else
    if type(items[idx][2]) == "table" then
      lcd.drawText(items[idx].x+panel.labelWidth,items[idx].y, items[idx][2][items[idx].value],flags+CUSTOM_COLOR)
    else
      lcd.drawText(items[idx].x+panel.labelWidth,items[idx].y, string.format(items[idx].fstring,items[idx].value,(items[idx][5]~=nil and items[idx][5] or "")),flags+CUSTOM_COLOR)
    end
  end
  lcd.drawText(items[idx].x+panel.columnWidth-5,items[idx].y, msgShortRequestStatus[items[idx].status],CUSTOM_COLOR+RIGHT+SMLSIZE)
end

local function drawListItem(items,idx)
  lcd.setColor(CUSTOM_COLOR,0xFFFF)    
  lcd.drawText(2,25 + (idx-menu.offset-1)*16, items[idx][1],CUSTOM_COLOR+SMLSIZE)
  if idx == menu.selectedItem then
    if menu.editSelected then
        flags = INVERS+BLINK
    else
      flags = INVERS
    end
  else
    flags = 0
  end
  
  if items[idx].value == nil then
    lcd.drawText(280,25 + (idx-menu.offset-1)*16, "--------",flags+CUSTOM_COLOR+SMLSIZE)
  else
    if type(items[idx][2]) == "table" then -- COMBO
      lcd.drawText(280,25 + (idx-menu.offset-1)*16, items[idx][2][items[idx].value],flags+CUSTOM_COLOR+SMLSIZE)
    else
      lcd.drawText(280,25 + (idx-menu.offset-1)*16, string.format(items[idx].fstring,items[idx].value,(items[idx][5]~=nil and items[idx][5] or "")),flags+CUSTOM_COLOR+SMLSIZE)
    end
  end
  lcd.drawText(LCD_W-2,25 + (idx-menu.offset-1)*16, msgRequestStatus[items[idx].status],flags+CUSTOM_COLOR+SMLSIZE+RIGHT)
end

local function writeItem(idxToSave)
  local item = (page <= #tuningPages and tuning[page].list[idxToSave] or params[page-#tuningPages].list[idxToSave])
  item.status = 2
  -- force this item as next to be processed
  idx = idxToSave
end

local function drawList(params,event)
  local items = params.list
  drawBars(items)
  if event == EVT_ENTER_BREAK then
    if menu.editSelected == true then
      -- confirm modified value
      writeItem(menu.selectedItem)
    else
      -- save last value for undo
      items[menu.selectedItem].lastValue = items[menu.selectedItem].value
    end
    if items[menu.selectedItem].value ~= nil then
      menu.editSelected = not menu.editSelected
      menu.updated = true
    else
      -- trigger refresh
      if items[menu.selectedItem].status == 0  or items[menu.selectedItem].status == 5 or items[menu.selectedItem].status == 4 then
        items[menu.selectedItem].status = 1
      end
    end
  elseif menu.editSelected and (event == EVT_EXIT_BREAK ) then
    items[menu.selectedItem].value = items[menu.selectedItem].lastValue
    menu.editSelected = not menu.editSelected
    menu.updated = false
  elseif menu.editSelected and (event == EVT_PLUS_BREAK or event == EVT_ROT_LEFT or event == EVT_PLUS_REPT) then
    incMenuItem(items,menu.selectedItem)
  elseif menu.editSelected and (event == EVT_MINUS_BREAK or event == EVT_ROT_RIGHT or event == EVT_MINUS_REPT) then
    decMenuItem(items,menu.selectedItem)
  elseif not menu.editSelected and (event == EVT_PLUS_BREAK or event == EVT_ROT_LEFT) then
    menu.selectedItem = (menu.selectedItem - 1)
    if menu.offset >=  menu.selectedItem then
      menu.offset = menu.offset - 1
    end
  elseif not menu.editSelected and (event == EVT_MINUS_BREAK or event == EVT_ROT_RIGHT) then
    menu.selectedItem = (menu.selectedItem + 1)
    if menu.selectedItem - 14 > menu.offset then
      menu.offset = menu.offset + 1
    end
  end
  --wrap
  if menu.selectedItem > #items then
    menu.selectedItem = 1 
    menu.offset = 0
  elseif menu.selectedItem  < 1 then
    menu.selectedItem = #items
    menu.offset =  math.max(0,#items - 14)
  end
  if params.panel == nil then
  -- draw list
    for m=1+menu.offset,math.min(#items,14+menu.offset) do
      lcd.setColor(CUSTOM_COLOR,0xFFFF)   
      drawListItem(items,m)
    end
  else
  -- draw list
    for m=1,#items do
      lcd.setColor(CUSTOM_COLOR,0xFFFF)   
      drawPanelItem(params,m)
    end
  -- draw boxes
    for b=1,#params.boxes do
      lcd.setColor(CUSTOM_COLOR,params.boxes[b].color)   
      lcd.drawRectangle(params.boxes[b].x,params.boxes[b].y,params.boxes[b].width,params.boxes[b].height,CUSTOM_COLOR)
      lcd.setColor(CUSTOM_COLOR,0x0000)   
      lcd.drawFilledRectangle(params.boxes[b].x+5,params.boxes[b].y-10,params.boxes[b].width-10,14,CUSTOM_COLOR)
      lcd.setColor(CUSTOM_COLOR,0xFFFF)   
      lcd.drawText(params.boxes[b].x+8,params.boxes[b].y-12,params.boxes[b].label,CUSTOM_COLOR+SMLSIZE)
    end
  end
  -- page title
  lcd.setColor(CUSTOM_COLOR,0xFFFF)
  lcd.drawText(0,0,string.format("%s - %d/%d",params.description, page, #paramsPages+#tuningPages),CUSTOM_COLOR)
end

local function compile()
  local files = {
    "mavlite",
    "default_params",
    
    "plane_tune",
    "qplane_tune",
    
    "copter_tune",
    "heli_tune",
    "rover_tune",
  }
  
  -- compile all layouts for all panes
  for i=1,#files do
    loadScript(libBasePath..files[i]..".lua","c")
  end
end

local function processMavliteMessage(msg)
  if msg.msgid == 23 then
    local param_name = mavLib.msg_get_name(msg,4)
    local param_value = mavLib.msg_get_value(msg,0)
  elseif msg.msgid == 22 then -- PARAM_VALUE
    local param_name = mavLib.msg_param_value_get_name(msg)
    local param_value = mavLib.msg_param_value_get_value(msg)

    utils.pushMessage(7,string.format("RX: ID=%d, %s : %f",msg.msgid,param_name,param_value))
    
    local item = getItemByName(globalParams,param_name)
    
    if item == nil then
      local paramsPage = (page <= #tuningPages and tuning[page] or params[page-#tuningPages])
      if paramsPage  ~= nil then
        item = getItemByName(page <= #tuningPages and tuning[page].list or params[page-#tuningPages].list,param_name)
      end
    end
    
    if item ~= nil then
      -- update value
      if type(item[2]) == "table" then -- COMBO
        -- look value up
        local success = false
        
        for i=1,#item[3]
        do
          if item[3][i] == param_value then
            item.value = i
            success = true
            break
          end
        end
        item.status = success == true and 5 or 6
      else
        item.value = param_value * (item.mult == nil and 1 or 1/item.mult)
        -- make all digits visible even if the increment has a lower resolution!
        -- when in panel mode precision is limited to 4 just like MP
        local precision = item.label == nil and 6 or 4
        
        item.fstring = "%.0"..tostring(math.min(precision,math.max(getDecimalCount(item.value),math.max(1,getDecimalCount(item[4]))))).."f %s"
        if item.value < item[2] or item.value > item[3] then
          -- update status
          item.status = 7
        else
          -- update status
          item.status = 5
        end
      end
    end
    
    collectgarbage()
    collectgarbage()
  end
end

local function formatMessage(severity,msg)
  local clippedMsg = msg
  
  if #msg > 50 then
    clippedMsg = string.sub(msg,1,50)
    msg = nil
    collectgarbage()
    collectgarbage()
  end
  
  if status.lastMessageCount > 1 then
    return string.format("%02d:%s (x%d) %s", status.messageCount, mavSeverity[severity], status.lastMessageCount, clippedMsg)
  else
    return string.format("%02d:%s %s", status.messageCount, mavSeverity[severity], clippedMsg)
  end
end

utils.getBitmap = function(name)
  if bitmaps[name] == nil then
    bitmaps[name] = Bitmap.open("/SCRIPTS/YAAPU/IMAGES/"..name..".png")
  end
  return bitmaps[name],Bitmap.getSize(bitmaps[name])
end

utils.unloadBitmap = function(name)
  if bitmaps[name] ~= nil then
    bitmaps[name] = nil
    -- force call to luaDestroyBitmap()
    collectgarbage()
    collectgarbage()
  end
end

utils.drawBlinkBitmap = function(bitmap,x,y)
  if blinkon == true then
      lcd.drawBitmap(utils.getBitmap(bitmap),x,y)
  end
end

utils.playSound = function(soundFile,skipHaptic)
  playFile(soundFileBasePath .."/"..conf.language.."/".. soundFile..".wav")
end

utils.pushMessage = function(severity, msg)
  if severity < 5 then
    utils.playSound("../err",true)
  else
    utils.playSound("../inf",true)
  end
  
  if msg == status.lastMessage then
    status.lastMessageCount = status.lastMessageCount + 1
  else  
    status.lastMessageCount = 1
    status.messageCount = status.messageCount + 1
  end
  if status.messages[(status.messageCount-1) % 20] == nil then
    status.messages[(status.messageCount-1) % 20] = {}
  end
  status.messages[(status.messageCount-1) % 20][1] = formatMessage(severity,msg)
  status.messages[(status.messageCount-1) % 20][2] = severity
  
  status.lastMessage = msg
  status.lastMessageSeverity = severity
  -- Collect Garbage
  collectgarbage()
  collectgarbage()
end

local function processTelemetry(sp)
  if sp.data_id == 0x5000 then -- MESSAGES
    if sp.value ~= status.lastMsgValue then
      status.lastMsgValue = sp.value
      local c
      local msgEnd = false
      for i=3,0,-1
      do
        c = bit32.extract(sp.value,i*8,7)
        if c ~= 0 then
          status.msgBuffer = status.msgBuffer .. string.char(c)
          collectgarbage()
          collectgarbage()
        else
          msgEnd = true;
          break;
        end
      end
      if msgEnd then
        local severity = (bit32.extract(sp.value,7,1) * 1) + (bit32.extract(sp.value,15,1) * 2) + (bit32.extract(sp.value,23,1) * 4)
        utils.pushMessage( severity, status.msgBuffer)
        status.msgBuffer = nil
        -- recover memory
        collectgarbage()
        collectgarbage()
        status.msgBuffer = ""
      end
    end
  elseif sp.data_id == 0x5007 then -- PARAMS
    paramId = bit32.extract(sp.value,24,4)
    paramValue = bit32.extract(sp.value,0,24)
    if paramId == 1 then -- frame type
      telemetry.frameType = paramValue
    end 
  end
end

local function processSportData(sportPacket)
  mavLib.process_sport_rx_data(mavlite_message, mavlite_status, processMavliteMessage, sportPacket)
end

local function createMsgParamRequestRead(paramName)
    
    local msg = {
      msgid = 20,
      len = 0,
      payload = {},
      checksum = 0
    }

    mavLib.msg_param_request_read_set_name(msg,paramName)
    
    collectgarbage()
    collectgarbage()
    
    return msg
end

local function createMsgParamSet(paramName, paramValue)
    
    local msg = {
      msgid = 23,
      len = 0,
      payload = {},
      checksum = 0
    }

    mavLib.msg_param_set_set_value(msg, paramValue)
    mavLib.msg_param_set_set_name(msg, paramName)
    
    collectgarbage()
    collectgarbage()
    
    return msg
end

local function initList(items)
  for idx=1,#items
  do
    if type(items[idx][2]) ~= "table" and items[idx].value ~= nil then
      -- make all digits visible even if the increment has a lower resolution!
      local precision = items[idx].label == nil and 6 or 4
      items[idx].fstring = "%.0"..tostring(math.min(precision,math.max(getDecimalCount(items[idx].value),math.max(1,getDecimalCount(items[idx][4]))))).."f %s"
      collectgarbage()
      collectgarbage()
    end
    -- initialize
    if items[idx].status == nil then
      items[idx].status = 1
      items[idx].timer = 0
    end
  end
end

--[[
  Process max MAX_ITEMS_PER_CYCLE items async
  idx is global so it passes all items in separate invocations to prevent cpu kill
--]]local function processItemsParamGet(items)
  local now = getTime()
  local itemsProcessed = 1
  while idx <= #items
  do
    -- check if a refresh is needed
    if items[idx].status == 1 then
      local msg = createMsgParamRequestRead(items[idx][1])
      --print("YAAPU:",msg.msgid,msg.len,string.char(msg.payload[0]),string.char(msg.payload[1]),string.char(msg.payload[2]),string.char(msg.payload[3]))
      if mavLib.msg_send(msg) == true then
        items[idx].status = 3
        items[idx].timer = getTime()
      end
      utils.clearTable(msg)
      msg = nil
    end
    idx = idx + 1
    if idx > #items then
      idx = 1
    end
    -- limit the number of items of this invocation to prevent a cpu kill
    itemsProcessed = itemsProcessed + 1
    if itemsProcessed > 1 then
      break
    end
  end
  collectgarbage()
  collectgarbage()
end

local function processItemsParamSet(items)
  for i=1,#items
  do
    -- check if a refresh is needed
    if items[i].status == 2 then
      if items[i].value ~= nil then
        local value = items[i].value * (items[i].mult == nil and 1 or items[i].mult)
        
        if type(items[i][2]) == "table" then
          value = items[i][3][items[i].value]
        end
        
        local msg = createMsgParamSet(items[i][1], value)
        --utils.pushMessage(7,"SET: "..items[i][1].." = "..tostring(value))
        if mavLib.msg_send(msg,utils) == true then
          items[i].status = 3
          items[i].timer = getTime()
        end
        utils.clearTable(msg)
        msg = nil
      end
    end
  end
  collectgarbage()
  collectgarbage()
end

local function processItemTimers(items)
  local now = getTime()
  for i=1,#items
  do
    -- check if a refresh is needed
    if items[i].status == 3 then 
      -- check timer
      if now - items[i].timer > 1000 then
        items[i].status = 4
        items[i].timer = now
      end
    elseif items[i].status == 4 then
      items[i].status = 1
    end
  end
end

local sendMavliteTimer = getTime()

local refreshTimer = 0

local function background()
  for i=1,20
  do
    local sensor_id,frame_id,data_id,value = sportTelemetryPop()
    
    if sensor_id  ~= nil then
      
      sportPacket.sensor_id = sensor_id
      sportPacket.frame_id = frame_id
      sportPacket.data_id = data_id
      sportPacket.value = value
      
      if sportPacket.frame_id == 0x10 then
        status.noTelemetryData = 0
        -- no telemetry dialog only shown once
        status.hideNoTelemetry = true
        
        processTelemetry(sportPacket)
      elseif sportPacket.frame_id == 0x32 then
        processSportData(sportPacket)
      end
    end  
  end
  
  
  --[[
  if getTime() - sendMavliteTimer > 25 then
    local msg = {
      msgid = 20,
      len = 0,
      payload = {},
      checksum = 0
    }
    
    mavLib.msg_param_request_read_set_name(msg,"Q_ENABLE")
    local success = mavLib.msg_send(msg)
    utils.pushMessage((success and 7 or 4),"TX: Q_ENABLE:"..(success and "OK" or "KO"))
    
    sendMavliteTimer = getTime()
  end
  --]]  
  if getTime() - refreshTimer > 25 then
    -- process global parameters
    processItemTimers(globalParams)
    processItemsParamGet(globalParams)
    
    -- process vehicle parameters
    if tuning[page] ~= nil then
        processItemTimers(tuning[page].list)
        processItemsParamGet(tuning[page].list)
        processItemsParamSet(tuning[page].list)
    elseif params[page-#tuningPages] ~= nil then
        processItemTimers(params[page-#tuningPages].list)
        processItemsParamGet(params[page-#tuningPages].list)
        processItemsParamSet(params[page-#tuningPages].list)
    end
    
    refreshTimer = getTime()
  end
  -- send pending packets
  mavLib.process_sport_tx_queue(utils)
  -- blinking support
  if (getTime() - blinktime) > 65 then
    blinkon = not blinkon
    blinktime = getTime()
  end
  collectgarbage()
  collectgarbage()
end


local function initFrameSpecificPages(frameName)
  -- look for frame specific pages
  local found = 1
  
  while found > 0 do
    local page = string.format("/SCRIPTS/YAAPU/CFG/%s_params_%d.lua",frameName,found)
    
    local file = io.open(page,"r")
    
    if file == nil then
      break
    end
    io.close(file)
    paramsPages[#paramsPages+1] = page
    utils.pushMessage(7,paramsPages[#paramsPages])
    found=found+1
  end
  collectgarbage()
  collectgarbage()
end

local searchFrameParams = true

local function loadFrameSpecificPages()
  if tuning[page] ~= nil then
    return
  end
  
  local frame = "plane"
  
  if telemetry.frameType ~= -1 then
    if frameTypes[telemetry.frameType] == "c" then
      tuningPages[1] = "copter_tune"
      frame = "copter"
    elseif frameTypes[telemetry.frameType] == "h" then
      tuningPages[1] = "heli_tune"
      frame = "heli"
    elseif frameTypes[telemetry.frameType] == "p" then
      local param = getItemByName(globalParams,"Q_ENABLE")
      
      if param == nil then
        globalParams[#globalParams+1] = {"Q_ENABLE"  , 0, 1, 1 ,status=1,timer=0}
      else
        if param.value ~= nil then
          if param.value > 0 then
            tuningPages[1] = "plane_tune"
            tuningPages[2] = "qplane_tune"
            frame = "qplane"
          else
            tuningPages[1] = "plane_tune"
            frame = "plane"
          end
        end
      end
    elseif frameTypes[telemetry.frameType] == "r" or frameTypes[telemetry.frameType] == "b" then
      tuningPages[1] = "rover_tune"
      frame = "rover"
    end
    
    if tuningPages[page] ~= nil then
      if page <= #tuningPages then 
        tuning[page] = utils.doLibrary(tuningPages[page])
      end
      
      if tuning[page].list then
        initList(tuning[page].list)
      end
      
      collectgarbage()
      collectgarbage()
      maxmem = 0
      
      if searchFrameParams == true then
        initFrameSpecificPages(frame)
        searchFrameParams = false
      end
    end
  end
end

local function getModelFilename()
  local info = model.getInfo()
  return "/SCRIPTS/YAAPU/CFG/" .. string.lower(string.gsub(info.name, "[%c%p%s%z]", ""))
end

local function createModelParamsFile()
  -- create the default model parameters file only if missing
  local pf = assert(io.open(getModelFilename().."_params_1.lua","a+"))
  if pf ~= nil then
    io.close(pf)
  end
end

local function initParamsPages()
  -- look for global frametype specific
  paramsPages[#paramsPages+1] = libBasePath.."default_params.lua"
  utils.pushMessage(7,paramsPages[#paramsPages])
  
  -- look for model specific pages
  local found = 1
  
  while found > 0 do
    local page = string.format("%s_params_%d.lua",getModelFilename(),found)
    
    local file = io.open(page,"r")
    
    if file == nil then
      break
    end
    local str = io.read(file,10)
    io.close(file)
    if #str == 0 then
      break
    end
    paramsPages[#paramsPages+1] = page
    utils.pushMessage(7,paramsPages[#paramsPages])
    found=found+1
  end
  collectgarbage()
  collectgarbage()
end

local function loadParamsPages()
  if params[page-#tuningPages] ~= nil then
    return
  end
  
  if page > #tuningPages then
    local p = loadScript(paramsPages[page-#tuningPages])
    if p == nil then
      params[page-#tuningPages] = nil
    else
      params[page-#tuningPages] = p()
    end
    collectgarbage()
    collectgarbage()
  end
  
  if params[page-#tuningPages].list then
    initList(params[page-#tuningPages].list)
  end
  
  maxmem = 0
end

local showMessageScreen = false
--------------------------
-- RUN
--------------------------
local function run(event)
  --[[
  if event > 0 then
    utils.pushMessage(7,tostring(event))
  end
  --]]  ------------------------
  -- CALC HUD REFRESH RATE
  ------------------------
  local hudnow = getTime()
  if hudcounter == 0 then
    hudstart = hudnow
  else
    hudrate = hudrate*0.8 + 100*(hudcounter/(hudnow - hudstart + 1))*0.2
  end
  hudcounter=hudcounter+1
  if hudnow - hudstart + 1 > 1000 then
    hudcounter = 0
  end
  background()
  
  lcd.setColor(CUSTOM_COLOR, 0x0AB1) -- hex 0x084c7b -- 073f66
  lcd.clear(CUSTOM_COLOR)
  ---------------------
  -- DRAW ITEMS
  ---------------------  
  if showMessageScreen then
    drawMessageScreen()
    
    if event == EVT_EXIT_BREAK or event == XLITE_RTN then
      showMessageScreen = false
    end
  else
    -- prevent page switch if frametype unknown
    if event == 513 and telemetry.frameType ~= -1 then
      
      utils.clearTable(params[page-#tuningPages])
      params[page-#tuningPages] = nil
      collectgarbage()
      collectgarbage()
      -- on page swithc clear tx queue
      mavLib.clear_tx_queue()
      
      page = page+1
      -- on page switch reset item counter
      idx = 1
      
      if page > (math.max(1,#paramsPages) + #tuningPages) then
        page = 1
      end
      
    end

    utils.drawBottomBar()

    if page <= #tuningPages or #tuningPages == 0 then
      if tuning[page] ~= nil then
        drawList(tuning[page], event)
      else
        if telemetry.frameType ~= -1 then
          drawWarning("...loading")
        else
          drawWarning("...getting vehicle type")
        end
        loadFrameSpecificPages()
      end
    else
      if params[page-#tuningPages] ~= nil then
        drawList(params[page-#tuningPages], event)
      else
        drawWarning("...loading")
        loadParamsPages()
      end
      subPage = page - #tuningPages
    end
    
    if event == 517 then
      showMessageScreen = true
    end
  
  end
  
  -- no telemetry/minmax outer box
  if telemetryEnabled() == false then
    utils.drawBlinkBitmap("warn",0,0)  
  end

  lcd.setColor(CUSTOM_COLOR,0xFE60)
  local hudrateTxt = string.format("%.1ffps",hudrate)
  lcd.drawText(480,3,hudrateTxt,SMLSIZE+CUSTOM_COLOR+RIGHT)
  lcd.setColor(CUSTOM_COLOR,lcd.RGB(255,0,0))
  maxmem = math.max(maxmem,collectgarbage("count")*1024)
  -- test with absolute coordinates
  lcd.drawNumber(450,LCD_H-14,maxmem,SMLSIZE+MENU_TITLE_COLOR+RIGHT)
  return 0
end

local function init()
  compile()
  -- if missing create a template file for parameters
  createModelParamsFile()
  
  -- load mavlite library
  mavLib = utils.doLibrary("mavlite")  
  
  -- look for general and model specific param pages
  initParamsPages()
    
  
  
  
  
  
  
  -- ok done
  utils.pushMessage(7,"Yaapu Ardupilot Lua 0.1-dev")
  
  collectgarbage()
  collectgarbage()
end

--------------------------------------------------------------------------------
-- SCRIPT END
--------------------------------------------------------------------------------
return {run=run, init=init}
